c	program ramabs
c
c     Program to calculate Raman and absorption intensities using time-dep
c     method with one excited electronic state treated in the simple harmonic
c	limit (no frequency changes or coordinate dependence of transition
c	moment).
c     Uses Mukamel's Brownian oscillator model for the solvent induced
c     broadening (identical to stochastic model for real part of broadening
c     function, but with imaginary part which gives solvent Stokes shift).
c     The formula correct when not in the high temperature limit is used.
c
c	Raman intensities are reported in units of differential cross section,
c	(A**2/molecule-sr)*1.e8
c
c	Absorption spectrum is printed in "absorp.txt"
c	Raman profile for line x is printed in "profx.txt"
c
c     Inputs (read from file "ramabs.in"):
c	  nstep = number of energies in calculated absorption spectrum and ex profiles
c     nmode = total # vibrational modes (30 max)
c     nline = # Raman lines to calculate (40 max)
c     ntime = # time steps in Fourier transform (5000 max)
c     e0 = electronic zero-zero energy 
c     gamma = electronic homogeneous linewidth (FWHM in cm-1)
c     rkappa = lineshape parameter in stochastic model
c     sig = electronic inhomogeneous width (Gaussian standard dev. in cm-1)
c     u = electronic transition length (A) 
c     alow,ahigh = lowest and highest energies to calc. absorption
c     delt = time step in Fourier transform (fs), typically around 0.5
c     refrac = solvent refractive index
c     wg(j) = vib. freq (cm-1) of mode j
c     delta(j) = dimensionless displacement of mode j 
c     temp = temperature in Kelvin
c     nquanta(i,j) = # of quanta excited in mode j in Raman line i
c	 
	subroutine ramabs(nmode, nline, ntime, nstep, sig, temp, e0, 
     1 gamma, rkappa, u, alow, ahigh, delt, refrac,wg,delta,nquanta,
     2 alife, spectrum, xfreq, xs)

     
      implicit real*8 (a-b), complex*8 (c), real*8 (d-h)
	implicit integer*4 (i-n), real*8 (o-z)
c	character*12 filen, ifile, ofile1, ofile2

c	input variables
	integer*4 nmode, nline, ntime, nstep, nquanta(40,30)
	real*8 sig, temp, e0, gamma, rkappa, u, alow, ahigh, 
     1 delt, refrac, alife 
	real*8 wg(30),delta(30)

c	output variables
      real*8 xfreq(1000),xs(1000)
	real*8 spectrum(1000,40)

	dimension covlp(30,5,5000),ceiwt(5000)
      dimension coswt(5000),csinwt(5000),cqt(5000),cpt(5000),
     1  cat(5000),cgt(5000),toverh(5000)
      dimension cdamp(5000),ffunc(1000),gfunc(1000),pfunc(501),
     1  rshift(40)
      real*8 vs(5000),vsi,arg,cutoff,einc,vn,dex,ainc
	character*4 base,numb(40),ext
      data pi/3.14159/ci/(0.,1.)/hbar/5308.8/
      data boltz/0.695/sq2/0.707107/sq32/1.224745/
	data base/'prof'/ext/'.txt'/cutoff/1.e-8/


	data numb/'01','02','03','04','05','06','07','08','09','10',
     1 '11','12','13','14','15','16','17','18','19','20','21','22',
	2 '23','24','25','26','27','28','29','30','31','32','33','34',
     3 '35','36','37','38','39','40'/

      pre = 2.08e-20*1.e-6*delt**2*0.3/pi
      delth = delt/hbar
      beta = 1./(boltz*temp)
      do 50 i=1,ntime
 50       toverh(i) = (i-1)*delth
c
c	Calculate vibrational stuff for both excited states
c	Brownian oscillator first
c
      do 55 i=1,ntime
 55      vs(i) = 0.
      rk2 = rkappa**2
      a = (2.355+1.76*rkappa)/(1.+0.85*rkappa+0.88*rk2)
      rlamb = rkappa*gamma/a
      reorg = beta*(rlamb/rkappa)**2/2.
      v = 2.*pi/beta
      n = 0
 510  n = n + 1
      vn = v*n*1.d0
      ainc=vn*delth
      arg=-1.d0
      einc=dexp(-ainc)
      dex=1.d0
      denom = vn*(vn**2-rlamb**2)
      ii = 0
      do 530 i=2,ntime
         arg = arg+ainc
         dex = dex*einc
         vsi = (dex+arg)/denom
         if(n.eq.1) go to 520
         if(dabs(vsi/vs(i)).gt.cutoff) ii=1
 520     vs(i) = vs(i) + vsi
 530     continue
      if(ii.ne.0.or.n.eq.1) go to 510
      vpre = 4.*reorg*rlamb/beta
      rpre = (reorg/rlamb)/dtan(rlamb*beta/2.0)
      do 60 i=1,ntime
          rlambt = rlamb*toverh(i)
          damp = 1. - dexp(-rlambt) - rlambt
          rdamp = -rpre*damp + vpre*vs(i) + alife*toverh(i)
 60       cdamp(i) = dconjg(cdexp(-rdamp+ci*reorg*damp/rlamb)) 
c	
c	Now undamped oscillators
c 
	do i=1,nmode
	    call simpov(i,delth,ntime,wg,delta,covlp,ceiwt)
		end do
c
c     Set up the time integrals
c
      do i=1,ntime
         cat(i)=u**2*cdexp(-ci*(e0+reorg)*toverh(i))*cdamp(i)
         end do
c
c	Set up inhomogeneous broadening
c
      part = 0.
	nhalf = nstep/2 + 1
      xfreq(1) = alow
      xinc = (ahigh-alow)/(nstep-1.)
      do 166 k=2,nstep
 166      xfreq(k) = xfreq(k-1) + xinc
      pfunc(1) = 0.
      do 605 i=2,nhalf
 605     pfunc(i) = pfunc(i-1) + xinc
      if(sig.eq.0.) go to 620
      deno = 2.*sig**2
      do 610 i=1,nhalf
 610      pfunc(i) = dexp(-pfunc(i)**2/deno)
c
c     Loop over all Raman lines to calculate. ceiwt is product of overlaps.
c
 620  do 288 j=1,nline
		do k=1,ntime
			ceiwt(k) = cat(k)
			end do
		rshift(j) = 0.
		do 140 k=1,nmode
			kk = nquanta(j,k)+1
			do l=1,ntime
				ceiwt(l)=ceiwt(l)*covlp(k,kk,l)
				end do
 140			rshift(j) = rshift(j) + (kk-1)*wg(k)
c
c     Loop over nstep excitation frequencies
c
		do 180 k=1,nstep
			csum1=0.5*ceiwt(1)
			cinc = cdexp(ci*xfreq(k)*delth)
			cold = (1.,0.)
c
c     Do time integral by simple sum (rectangle rule)
c
			do 170 l=2,ntime
				cold = cold*cinc
 170              csum1 = csum1 + cold*ceiwt(l)
			csigma = csum1             
c
c     Calculate total cross sections (k-th excitation freq.,
c              j-th Raman mode)
c
               sigma = csigma*dconjg(dcmplx(csigma))                      
 180           spectrum(k,j) = sigma + spectrum(k,j)
 288		  continue
c
c     Convolve with Gaussian inhomogeneous distribution
c
      do 700 i=1,nline
		do j=1,nstep
			ffunc(j) = spectrum(j,i)
		end do
          call convl(ffunc,pfunc,gfunc,nstep,nhalf,sig)
          do j=1,nstep
			spectrum(j,i)=pre*gfunc(j)*xfreq(j)*(xfreq(j)
     1              -rshift(i))**3
		end do
 700      continue
c
c     Print out profiles
c
	do i=1,nline
		do j=1,nstep
			spectrum(j,i)=spectrum(j,i)*1.e8
			end do
		end do
c      do 350 i=1,nline
c		open(unit=31+i,name=base//numb(i)//ext)
c          write (31+i,341) (xfreq(j),spectrum(j,i),j=1,nstep)
c 341      format(1x,f11.3,5x,f11.4)
c 350      close(unit=31+i)
c
c     Calculate absorption spectrum
c
 352  pre2 = 5.745e-3*1.e-3*delt/refrac
	do i=1,ntime
		ceiwt(i) = cat(i)
		end do
      do k=1,nmode
		do l=1,ntime
			ceiwt(l) = ceiwt(l)*covlp(k,1,l)
			end do
		end do
      do 450 k=1,nstep
		csum = 0.5*ceiwt(1)
		cinc = cdexp(ci*xfreq(k)*delth)
		cold = (1.,0.)
		do l=2,ntime
			cold = cold*cinc
			csum=csum+cold*ceiwt(l)
			end do
		sigma = dabs(dble(csum))*dble(pre2)
 450		xs(k) = sigma + xs(k)
      call convl(xs,pfunc,gfunc,nstep,nhalf,sig)
      do j=1,nstep
		xs(j) = gfunc(j)*xfreq(j)
		end do
      open(unit=31,name='simpov.txt')
c     do 731 i=1,nstep
c731		write (31,470) xfreq(i),xs(i)
c470  format(2f21.14)
c     close(unit=31)
c      stop
      end
c
c
      subroutine convl(f,p,g,nf,np,sig)
c
c     calculates the convolution of f with p and returns result in g
c
	implicit complex*8 (c)
	implicit real*8 (a-b, d-h, o-z), integer*4 (i-j)
      dimension f(nf),p(np),g(nf)
c
      if(sig.eq.0.) go to 500
      pmult = p(1)
      sum = pmult
      do 100 i=1,nf
 100      g(i) = f(i)*pmult
      do 300 j=1,np-1
          pmult = p(j+1)
          if(pmult.lt.1.e-5) go to 350
          do 200 k=1,nf
        kmj = k - j
        if(kmj.gt.0) g(k)=g(k)+pmult*f(kmj)
        kpj = k + j
 200    if(kpj.le.nf) g(k)=g(k)+pmult*f(kpj)
 300      sum = sum + 2.*pmult
 350  do 400 i=1,nf
 400      g(i) = g(i)/sum
      return
 500  do 510 i=1,nf
 510      g(i) = f(i)
      return
      end
c
c
c
      subroutine simpov(n,delth,ntime,wg,delta,covlp,ceiwt)
c
c     Calculates time-dependent overlap for mode with equal ground and
c     excited state frequency
c
      implicit complex*8 (c)
	implicit real*8 (a-b, d-h, o-z), integer*4 (i-j)
      dimension wg(30),delta(30),covlp(30,5,5000),ceiwt(5000)
      data ci/(0.,1.)/sq2/0.7071/sq6/2.44949/
c
      s = delta(n)**2/2.
	sqrts = -delta(n)*sq2
      cinc = cdexp(-ci*wg(n)*delth)
      ceiwt(1) = (1.,0.)
      do 10 i=2,ntime
 10       ceiwt(i) = ceiwt(i-1)*cinc
c
c     Calculate <0|0(t)> through <4|0(t)>
c
      do 30 i=1,ntime
          ce = ceiwt(i)
          ce1 = 1. - ce
          ct = cdexp(-s*ce1)
          covlp(n,1,i)=cdexp(-s*(1.-ceiwt(i)))
          covlp(n,2,i)=sqrts*(ceiwt(i)-1.)*covlp(n,1,i)
          covlp(n,3,i)=sq2*s*(ceiwt(i)-1.)**2*covlp(n,1,i)
          covlp(n,4,i)=-s*sqrts*(1.-ceiwt(i))**3*covlp(n,1,i)/sq6
 30       covlp(n,5,i)=s**2*(1.-ceiwt(i))**4/(2.*sq6)
      return
      end
c
c
	subroutine mexFunction(nlhs, plhs, nrhs, prhs)
c
c	function to export ramabs() to Matlab
c	
c	Might in the future use this function to be the automater....
c
c	nlhs is the number of expected ouputs
c	plhs is the Array of pointers to outputs
c	nrhs is the number of inputs
c	prhs is the Array of pointers to inputs	

c	implicit none

	integer mxGetM, mxGetN, mxGetPr, mxIsNumeric
      integer mxCreateDoubleMatrix
	integer plhs(*), prhs(*)
	integer nlhs, nrhs, m, n, size
c	input variables
	integer*4 nmode, nline, ntime, nstep, nquanta(40,30)
	real*8 sig, temp, e0, gamma, rkappa, u, alow, ahigh, delt, refrac 
	real*8 wg(30),delta(30), alife


c	input pointers


	integer*8 nmode_ptr, nline_ptr, ntime_ptr, nstep_ptr, nquanta_ptr
	integer*8 sig_ptr, temp_ptr, e0_ptr, gamma_ptr, rkappa_ptr, u_ptr
	integer*8 alow_ptr, ahigh_ptr, delt_ptr, refrac_ptr
	integer*8 wg_ptr, delta_ptr, alife_ptr

c	output variables
      real*8 xfreq(1000),xs(1000), spectrum(1000,40)

c	output pointers
	integer xfreq_ptr, xs_ptr, spectrum_ptr
	

C     Check for proper number of arguments. 
      if(nrhs .ne. 18) then
        call mexErrMsgTxt('18 inputs required.')
      elseif(nlhs .ne. 3) then
         call mexErrMsgTxt('3 outputs required.')
      endif


	size = 1

	nmode_ptr = mxGetPr(prhs(1))
	call mxCopyPtrToInteger4(nmode_ptr, nmode, size)

	nline_ptr = mxGetPr(prhs(2))
	call mxCopyPtrToInteger4(nline_ptr, nline, size)

	ntime_ptr = mxGetPr(prhs(3))
	call mxCopyPtrToInteger4(ntime_ptr, ntime, size)

	nstep_ptr = mxGetPr(prhs(4))
	call mxCopyPtrToInteger4(nstep_ptr, nstep, size) 

	sig_ptr = mxGetPr(prhs(5))
	call mxCopyPtrToReal8(sig_ptr, sig, size)

	temp_ptr = mxGetPr(prhs(6))
	call mxCopyPtrToReal8(temp_ptr, temp, size)
	
	e0_ptr = mxGetPr(prhs(7))
	call mxCopyPtrToReal8(e0_ptr, e0, size)

	gamma_ptr = mxGetPr(prhs(8))
	call mxCopyPtrToReal8(gamma_ptr, gamma, size)

	rkappa_ptr = mxGetPr(prhs(9))
	call mxCopyPtrToReal8(rkappa_ptr, rkappa, size)

	u_ptr = mxGetPr(prhs(10))
	call mxCopyPtrToReal8(u_ptr, u, size)

	alow_ptr = mxGetPr(prhs(11))
	call mxCopyPtrToReal8(alow_ptr, alow, size)

	ahigh_ptr = mxGetPr(prhs(12))
	call mxCopyPtrToReal8(ahigh_ptr, ahigh, size)

	delt_ptr = mxGetPr(prhs(13))
	call mxCopyPtrToReal8(delt_ptr, delt, size)

	refrac_ptr = mxGetPr(prhs(14))
	call mxCopyPtrToReal8(refrac_ptr, refrac, size)

	m = mxGetM(prhs(15))
      n = mxGetN(prhs(15))
	wg_ptr = mxGetPr(prhs(15))
	size = m*n
	call mxCopyPtrToReal8(wg_ptr, wg, size)

	m = mxGetM(prhs(16))
      n = mxGetN(prhs(16))
	delta_ptr = mxGetPr(prhs(16))
	call mxCopyPtrToReal8(delta_ptr, delta, size)
	

	m = mxGetM(prhs(17))
      n = mxGetN(prhs(17))
	nquanta_ptr = mxGetPr(prhs(17))
	size = m*n
	call mxCopyPtrToInteger4(nquanta_ptr, nquanta, size)

      alife_ptr = mxGetPr(prhs(18))
	  call mxCopyPtrToReal8(alife_ptr, alife, 1)

	if( nquanta(1,1) .gt. 10 ) then
		call mexErrMsgTxt('Nquanta can not be greater than 10. Please 
     1 check the input file')
      endif 	 
	
	do i = 1,1000
		xs(i) = 0
		xfreq(i) = 0
	end do

	do i = 1,1000
		do j = 1,40
			spectrum(i,j) = 0
		end do
	end do

c processing
     
	call ramabs(nmode, nline, ntime, nstep, sig, temp, e0, 
     1 gamma, rkappa, u, alow, ahigh, delt, refrac,wg,delta,nquanta,
     2 alife, spectrum, xfreq, xs)
	
c	outputting data

	plhs(1) = mxCreateDoubleMatrix(1000,nline, 0)
	spectrum_ptr = mxGetPr(plhs(1))	
	call mxCopyReal8ToPtr(spectrum, spectrum_ptr, 1000*nline)

	plhs(2) = mxCreateDoubleMatrix(nstep,1, 0)
	xs_ptr = mxGetPr(plhs(2))
	call mxCopyReal8toPtr(xs, xs_ptr,nstep)

	plhs(3) = mxCreateDoubleMatrix(nstep,1, 0)
	xfreq_ptr = mxGetPr(plhs(3))
	call mxCopyReal8toPtr(xfreq, xfreq_ptr,nstep)
	
	end subroutine

